indexing

	description:

		"Single inlet, multiple outlets. Write each element to all outlets,%
		%in order of connection."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2001, Simon Parker and others"
	license: "Eiffel Forum License v1 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"


class

	PW_DRAIN_SPLITTER [INWARD]


inherit

	PW_WRITER [INWARD]
		redefine
			flush
		end


creation

	make,
	connect


feature -- Initialization

	make is
			-- Create or reset the splitter.
		do
			!! drains.make
		ensure then
			not_connected: not is_connected
			drains_not_void: drains /= Void
			drains_empty: drains.is_empty
		end

feature -- Status

	is_connected: BOOLEAN is
			-- Is there at least one outlet?
		do
			Result := drains.count > 0
		end

	is_outlet_connected (outlet: like drain_anchor): BOOLEAN is
			-- Has `connect' been called for the given `outlet'?
		do
			Result := drains.has (outlet)
		end

feature -- Connection management

	connect (outlet: like drain_anchor) is
			-- Add (another reference to) outlet.
		require
			outlet_given: outlet /= Void
			outlet_not_connected: not is_outlet_connected (outlet)
		do
			if drains = Void then
				make
			end
			drains.put_last (outlet)
		ensure
			outlet_connected: is_outlet_connected (outlet)
			connected: is_connected
		end

	disconnect (outlet: like drain_anchor) is
			-- Remove `outlet' from drains to write to.
		require
			outlet_given: outlet /= Void
			outlet_connected: is_outlet_connected (outlet)
		do
			drains.delete (outlet)
		ensure
			outlet_not_connected: not is_outlet_connected (outlet)
		end

feature -- Writing

	put (item: INWARD) is
			-- Write `item' to each outlet, if any. Stops as soon as an
			-- error occurs.
		do
			from
				drains.start
			until
				drains.after or else
				not drains.item_for_iteration.is_ok
			loop
				drains.item_for_iteration.put (item)
				if drains.item_for_iteration.is_ok then
					drains.forth
				end
			end
			if not drains.after and then not drains.item_for_iteration.is_ok then
				diagnostic_code := drains.item_for_iteration.diagnostic_code
				diagnostic_message := drains.item_for_iteration.diagnostic_message
			end
		end


feature -- Flushing

	flush is
			-- Flush each outlet.
		do
			from
				drains.start
			until
				drains.after or else
				not drains.item_for_iteration.is_ok
			loop
				drains.item_for_iteration.flush
				if drains.item_for_iteration.is_ok then
					drains.forth
				end
			end
			if not drains.after and then not drains.item_for_iteration.is_ok then
				diagnostic_code := drains.item_for_iteration.diagnostic_code
				diagnostic_message := drains.item_for_iteration.diagnostic_message
			end
		end


feature {NONE} -- Implementation

	drain_anchor: PW_WRITER [INWARD]
			-- Type to make it possible to be more specific about outlets.

	drains: DS_LINKED_LIST [like drain_anchor]
			-- Connected outlets


invariant

	list_exists: drains /= Void

end
