indexing

	description:

		"Accept INWARDs and write OUTWARDs, not necessarily one for one."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2001, Simon Parker and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"


deferred class

	PW_DRAIN_WRITER [INWARD]

inherit

	PW_DESTINATION_WRITER [INWARD]
		redefine
			clear_error,
			drain,
			flush
		end

feature -- Error handling

	clear_error is
		do
			precursor
			if drain /= Void then
				drain.clear_error
			end
		end

	copy_error_from_drain is
			-- If not `drain'.`is_ok', it's error code and diagnostic
			-- message is copied. Else nothing happens.
		do
			if not drain.is_ok then
				diagnostic_code := drain.diagnostic_code
				diagnostic_message := drain.diagnostic_message
			end
		ensure
			error_copied: not drain.is_ok implies
				(diagnostic_code = drain.diagnostic_code and
				 diagnostic_message.is_equal (drain.diagnostic_message))
			not_copied: drain.is_ok implies
				diagnostic_code = old diagnostic_code
		end

feature -- Connection management

	drain: PW_WRITER [ANY]
			-- Where we send our output.
			-- Redefine for more precise type.

feature -- Writing

	flush is
			-- Flush myself first, then drain.
		do
			flush_current
			if is_connected then
				drain.flush
			end
		end

	put (item: INWARD) is
			-- Dispose of 'item'. This is the default implementation,
			-- usually it is overriden.
		do
			drain.put (item)
			copy_error_from_drain
		end

end
