indexing

	description:

		"A source of zero or more objects of a given type."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2001, Simon Parker and others"
	license: "Eiffel Forum License v1 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"


deferred class

	PW_READER [OUTWARD]

inherit

	KI_INPUT_STREAM [OUTWARD]
		redefine
			is_rewindable,
			rewind,
			valid_unread_item
		end

	PW_LAST_RESULT

feature -- Status

	frozen is_at_end: BOOLEAN is
			-- Are there no more objects to read?
		obsolete "2003-10-23: Use end_of_input instead."
		do
			Result := is_at_end
		end

	frozen is_last_read_valid: BOOLEAN is
			-- Was `read' successful, so does `last_read' contain
			-- something valid?
		obsolete "2003-10-23: When you use last_item, this routine is no longer applicable."
		do
			Result := is_open_read and not end_of_input
		end

	is_rewindable: BOOLEAN is
			-- Can current input stream be rewound to return input from
			-- the beginning of the stream?
		do
			Result := is_open_read
		end

	valid_unread_item (an_item: OUTWARD): BOOLEAN is
			-- Can `an_item' be put back in input stream?
			-- Always false for PW_READERs.
		do
			Result := False
		ensure then
			nope: not Result
		end


feature -- Access

	name: STRING is
			-- Name of input stream
		do
			-- not applicable.
			Result := ""
		end

feature -- Reading

	frozen last_read: OUTWARD is
			-- Object obtained by most recent 'read' call
		obsolete "2003-10-23: Use last_item instead."
-- 		require
-- 			read_was_called: is_last_read_valid
		do
			Result := last_item
		end

	frozen prime is
			-- Prepare for `read'. Postcondition is precondition of
			-- `read'.
		obsolete "2003-10-23: Use rewind instead."
		do
			rewind
		end

	read is
			-- Read the next item in input stream.
			-- Make the result available in `last_item'.
			-- If an error occurs, `end_of_input' is set.
		deferred
		ensure then
			end_of_input_on_error: not is_ok implies end_of_input
		end

	rewind is
		do
			-- nothing
		ensure then
			ok_is_something_to_read: is_ok = not end_of_input
		end

	unread (an_item: OUTWARD) is
		do
			-- nothing
		end


end
