indexing

	description:

		"Obtain characters from a source of strings."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2002, Simon Parker and others"
	license: "Eiffel Forum License v1 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"

class

	PW_STRING_CHARACTER_READER

inherit

	PW_SOURCE_READER [CHARACTER]
		redefine
			source
		end

creation

	make,
	connect

feature -- Status

	is_at_end: BOOLEAN is
			-- Are there no more characters?
		do
			Result := is_connected
				and then is_primed
				and then source.is_at_end
		end

feature -- Reading

	read is
			-- Try to read another character.
		do
			if not is_primed then
				source.read
				index := 1
				is_primed := true
			else
				index := index + 1
			end
			-- Skip empty lines...
			if index > source.last_read.count then
				from
					source.read
					index := 1
				until
					source.is_at_end or else
					source.last_read.count > 0
				loop
					source.read
				end
			end
		end

	last_read: CHARACTER is
			-- Character obtained by most recent 'read' call.
		do
			Result := source.last_read.item (index)
		end

feature -- Connection

	source: READER [STRING]
			-- Source of lines

feature {} -- Internal controls

	index: INTEGER
			-- Position of last character read in current source string

invariant

	index_ok_if_ready: (is_primed and not is_at_end)
	implies index <= source.last_read.count

end
